@@ -28,7 +28,10 @@ import select as oselect
 from os import waitpid as owaitpid
 import xml.etree.ElementTree as XET
 from select import error as SelectError
-from cPickle import PickleError
+try:
+    from cPickle import PickleError
+except ImportError:
+    from pickle import PickleError
 
 from conf import GLUSTERFS_LIBEXECDIR, UUID_FILE
 sys.path.insert(1, GLUSTERFS_LIBEXECDIR)
@@ -62,6 +65,8 @@ CHANGELOG_AGENT_SERVER_VERSION = 1.0
 CHANGELOG_AGENT_CLIENT_VERSION = 1.0
 NodeID = None
 rsync_version = None
+unshare_mnt_propagation = None
+slv_bricks = None
 SPACE_ESCAPE_CHAR = "%20"
 NEWLINE_ESCAPE_CHAR = "%0A"
 PERCENTAGE_ESCAPE_CHAR = "%25"
@@ -155,7 +160,8 @@ def setup_ssh_ctl(ctld, remote_addr, resource_url):
     rconf.ssh_ctl_dir = ctld
     content = "SLAVE_HOST=%s\nSLAVE_RESOURCE_URL=%s" % (remote_addr,
                                                         resource_url)
-    content_sha256 = sha256hex(content)
+    encoded_content = content.encode()
+    content_sha256 = sha256hex(encoded_content)
     """
     The length of ctl_path for ssh connection should not be > 108.
     ssh fails with ctl_path too long if it is so. But when rsync
@@ -167,7 +173,7 @@ def setup_ssh_ctl(ctld, remote_addr, resource_url):
     fname = os.path.join(rconf.ssh_ctl_dir,
                          "%s.mft" % content_sha256)
 
-    create_manifest(fname, content)
+    create_manifest(fname, encoded_content)
     ssh_ctl_path = os.path.join(rconf.ssh_ctl_dir,
                                 "%s.sock" % content_sha256)
     rconf.ssh_ctl_args = ["-oControlMaster=auto", "-S", ssh_ctl_path]
@@ -180,7 +186,7 @@ def grabfile(fname, content=None):
     """
     # damn those messy open() mode codes
     fd = os.open(fname, os.O_CREAT | os.O_RDWR)
-    f = os.fdopen(fd, 'r+b', 0)
+    f = os.fdopen(fd, 'r+')
     try:
         fcntl.lockf(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
     except:
@@ -194,6 +200,7 @@ def grabfile(fname, content=None):
         try:
             f.truncate()
             f.write(content)
+            f.flush()
         except:
             f.close()
             raise
@@ -256,7 +263,8 @@ def finalize(*args, **kwargs):
             umount_cmd = rconf.mbr_umount_cmd + [rconf.mount_point, 'lazy']
         else:
             umount_cmd = ['umount', '-l', rconf.mount_point]
-        p0 = subprocess.Popen(umount_cmd, stderr=subprocess.PIPE)
+        p0 = subprocess.Popen(umount_cmd, stderr=subprocess.PIPE,
+                              universal_newlines=True)
         _, errdata = p0.communicate()
         if p0.returncode == 0:
             try:
@@ -623,6 +631,23 @@ def get_master_and_slave_data_from_args(args):
 
     return (master_name, slave_data)
 
+def unshare_propagation_supported():
+    global unshare_mnt_propagation
+    if unshare_mnt_propagation is not None:
+        return unshare_mnt_propagation
+
+    unshare_mnt_propagation = False
+    p = subprocess.Popen(["unshare", "--help"],
+                         stderr=subprocess.PIPE,
+                         stdout=subprocess.PIPE,
+                         universal_newlines=True)
+    out, err = p.communicate()
+    if p.returncode == 0:
+        if "propagation" in out:
+            unshare_mnt_propagation = True
+
+    return unshare_mnt_propagation
+
 
 def get_rsync_version(rsync_cmd):
     global rsync_version
@@ -632,7 +657,8 @@ def get_rsync_version(rsync_cmd):
     rsync_version = "0"
     p = subprocess.Popen([rsync_cmd, "--version"],
                          stderr=subprocess.PIPE,
-                         stdout=subprocess.PIPE)
+                         stdout=subprocess.PIPE,
+                         universal_newlines=True)
     out, err = p.communicate()
     if p.returncode == 0:
         rsync_version = out.split(" ", 4)[3]
@@ -640,6 +666,40 @@ def get_rsync_version(rsync_cmd):
     return rsync_version
 
 
+def get_slv_dir_path(slv_host, slv_volume, gfid):
+    global slv_bricks
+
+    dir_path = ENOENT
+
+    if not slv_bricks:
+        slv_info = Volinfo(slv_volume, slv_host)
+        slv_bricks = slv_info.bricks
+    # Result of readlink would be of format as below.
+    # readlink = "../../pgfid[0:2]/pgfid[2:4]/pgfid/basename"
+    for brick in slv_bricks:
+        dir_path = errno_wrap(os.path.join,
+                              [brick['dir'],
+                               ".glusterfs", gfid[0:2],
+                               gfid[2:4],
+                               gfid], [ENOENT], [ESTALE])
+        if dir_path != ENOENT:
+            break
+
+    if not isinstance(dir_path, int):
+        realpath = errno_wrap(os.readlink, [dir_path],
+                              [ENOENT], [ESTALE])
+
+        if not isinstance(realpath, int):
+            realpath_parts = realpath.split('/')
+            pargfid = realpath_parts[-2]
+            basename = realpath_parts[-1]
+            pfx = gauxpfx()
+            dir_entry = os.path.join(pfx, pargfid, basename)
+            return dir_entry
+
+    return None
+
+
 def lf(event, **kwargs):
     """
     Log Format helper function, log messages can be
@@ -787,7 +847,7 @@ class Popen(subprocess.Popen):
                 break
             b = os.read(self.stderr.fileno(), 1024)
             if b:
-                elines.append(b)
+                elines.append(b.decode())
             else:
                 break
         self.stderr.close()
@@ -801,7 +861,7 @@ class Volinfo(object):
     def __init__(self, vol, host='localhost', prelude=[]):
         po = Popen(prelude + ['gluster', '--xml', '--remote-host=' + host,
                               'volume', 'info', vol],
-                   stdout=PIPE, stderr=PIPE)
+                   stdout=PIPE, stderr=PIPE, universal_newlines=True)
         vix = po.stdout.read()
         po.wait()
         po.terminate_geterr()
@@ -880,7 +940,7 @@ class VolinfoFromGconf(object):
     # Glusterd will generate following config items before Geo-rep start
     # So that Geo-rep need not run gluster commands from inside
     # Volinfo object API/interface kept as is so that caller need not
-    # change anything exept calling this instead of Volinfo()
+    # change anything except calling this instead of Volinfo()
     #
     # master-bricks=
     # master-bricks=NODEID:HOSTNAME:PATH,..
